import { h } from '../anjuna-charts.core.js';

import './chunk-1ad07b08.js';
import { a as ChartService, b as SeriesStyle, c as ChartDataService } from './chunk-ed6b5d08.js';

class XAxisStrategy {
    constructor(emitter) {
        this.emitter = emitter;
        this.fallBackType = 'date';
    }
    addAxis(svg, dims, data, axisConfigs) {
        return ChartService.addXAxis(svg, dims, data, axisConfigs);
    }
    emit(axisConfig) {
        return this.emitter.emit(axisConfig);
    }
}
class YAxisStrategy {
    constructor(emitter) {
        this.emitter = emitter;
        this.fallBackType = 'string';
    }
    addAxis(svg, dims, data, axisConfigs) {
        return ChartService.addYAxis(svg, dims, data, axisConfigs);
    }
    emit(axisConfig) {
        return this.emitter.emit(axisConfig);
    }
}

class Axis {
    constructor() {
        this.dimension = 'x';
        this.show = true;
        this.ticks = true;
    }
    componentDidLoad() {
        this.service = (this.dimension === 'x') ? new XAxisStrategy(this.anjXAxisDidLoad) : new YAxisStrategy(this.anjYAxisDidLoad);
        const axisConfig = {
            showAxis: this.show,
            showTicks: this.ticks,
            labelType: this.type || this.service.fallBackType,
            formattingOptions: {
                locale: this.locale || 'en-US',
                language: this.language || 'en',
                currency: this.currency || 'USD'
            }
        };
        this.service.emit(axisConfig);
    }
    async init(svg, dims, data, axisConfigs) {
        return Promise.resolve(this.service.addAxis(svg, dims, data, axisConfigs));
    }
    static get is() { return "anj-axis"; }
    static get properties() { return {
        "currency": {
            "type": String,
            "attr": "currency"
        },
        "dimension": {
            "type": String,
            "attr": "dimension",
            "reflectToAttr": true
        },
        "element": {
            "elementRef": true
        },
        "init": {
            "method": true
        },
        "language": {
            "type": String,
            "attr": "language"
        },
        "locale": {
            "type": String,
            "attr": "locale"
        },
        "show": {
            "type": Boolean,
            "attr": "show"
        },
        "ticks": {
            "type": Boolean,
            "attr": "ticks"
        },
        "type": {
            "type": String,
            "attr": "type"
        }
    }; }
    static get events() { return [{
            "name": "anjYAxisDidLoad",
            "method": "anjYAxisDidLoad",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }, {
            "name": "anjXAxisDidLoad",
            "method": "anjXAxisDidLoad",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }]; }
}

class Line {
    componentDidLoad() {
        this.anjLineDidLoad.emit();
    }
    async init(svg, x, y, data, index, dims, multiSeries) {
        this.color = this.color || ChartService.colors[index || 0];
        const line = ChartService.addLine(svg, x, y, data, this.color);
        if (multiSeries) {
            ChartService.addScrubbers(svg, x, data, dims, index);
        }
        else {
            ChartService.addArea(svg, line, y, data, this.color);
        }
        ChartService.addPoints(svg, x, y, data, this.color, index);
        return Promise.resolve(this.color);
    }
    static get is() { return "anj-line"; }
    static get properties() { return {
        "color": {
            "type": String,
            "attr": "color",
            "mutable": true
        },
        "dataIndex": {
            "type": Number,
            "attr": "data-index"
        },
        "element": {
            "elementRef": true
        },
        "init": {
            "method": true
        },
        "name": {
            "type": String,
            "attr": "name"
        },
        "netNegative": {
            "type": Boolean,
            "attr": "net-negative"
        }
    }; }
    static get events() { return [{
            "name": "anjLineDidLoad",
            "method": "anjLineDidLoad",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }]; }
}

class SingleLineSeries {
    flatten(data) {
        return data;
    }
    getPopoverEl(svg, index) {
        return svg.select('.point-' + index).nodes()[0];
    }
    getPopoverPlacement() {
        return 'top';
    }
    getSeriesStyle() {
        return SeriesStyle.Single;
    }
    getTargetingData(data) {
        return data;
    }
    isMultiSeries() {
        return false;
    }
}
class MultiLineSeries {
    flatten(data) {
        return [].concat(...data);
    }
    getPopoverEl(svg, index) {
        return svg.select('.scrubber-' + index).nodes()[0];
    }
    getPopoverPlacement() {
        return 'right';
    }
    getSeriesStyle() {
        return SeriesStyle.Multi;
    }
    getTargetingData(data) {
        return data[0];
    }
    isMultiSeries() {
        return true;
    }
}

class LineChart {
    constructor() {
        this.service = new SingleLineSeries();
    }
    dataSourceHandler(newValue, oldValue) {
        if (newValue !== oldValue) {
            this.debounceCreation();
        }
    }
    componentWillLoad() {
        this.axisConfigs = {
            x: this.getDefaultXAxisConfig(),
            y: this.getDefaultYAxisConfig()
        };
    }
    componentDidLoad() {
        this.debounceCreation();
    }
    async onLinesLoaded() {
        this.lines = await this.loadLines();
        this.debounceCreation();
    }
    async onXAxisLoaded({ detail }) {
        this.xAxisElement = await this.loadXAxis();
        this.axisConfigs.x = detail;
        this.debounceCreation();
    }
    async onYAxisLoaded({ detail }) {
        this.yAxisElement = await this.loadYAxis();
        this.axisConfigs.y = detail;
        this.debounceCreation();
    }
    anjEndHoverHandler(svg) {
        window.clearTimeout(this.popoverTimer);
        this.activeIndex = null;
        this.popover.close();
        ChartService.hideFocus(svg);
    }
    anjHoverHandler(svg, data, x) {
        const targetData = this.service.getTargetingData(data);
        const dataIndex = ChartService.getIndexOfClosestPoint(x, targetData);
        let activeData;
        if (this.activeIndex !== dataIndex) {
            this.activeIndex = dataIndex;
            const seriesStyle = this.service.getSeriesStyle();
            activeData = { data, axisConfigs: this.axisConfigs, activeIndex: this.activeIndex, seriesStyle, seriesMeta: this.seriesMeta };
            this.anjHover.emit(activeData);
            this.popoverTimer = window.setTimeout(() => {
                const popoverEl = this.service.getPopoverEl(svg, this.activeIndex);
                this.display(popoverEl);
                ChartService.showFocus(svg, this.activeIndex);
            }, LineChart.popoverDebounceLength);
        }
    }
    addEvents(svg, chartOverlay, x, data) {
        chartOverlay
            .on('mousemove', () => this.anjHoverHandler(svg, data, x))
            .on('mouseout', () => this.anjEndHoverHandler(svg));
    }
    appendSVG(svg) {
        while (this.chartContainer.children.length > 1) {
            this.chartContainer.removeChild(this.chartContainer.lastChild);
        }
        this.chartContainer.appendChild(svg.node());
    }
    debounceCreation() {
        window.clearTimeout(this.creationTimer);
        this.creationTimer = window.setTimeout(async () => {
            this.chartSVG = await this.create();
            console.log(this.chartSVG);
        }, LineChart.creationDebounceLength);
    }
    async display(target) {
        const popper = await this.popover.popper();
        if (popper) {
            popper.reference = target;
            popper.scheduleUpdate();
        }
        else {
            this.popover.open(target);
        }
    }
    async create() {
        const data = this.handleDataSource();
        const dims = this.getDims();
        const flattenedData = this.service.flatten(data);
        const svg = ChartService.createSVG(dims);
        const x = (this.xAxisElement)
            ? await this.hydrateXAxis(dims, flattenedData, svg)
            : ChartService.addXAxis(svg, dims, flattenedData, this.axisConfigs);
        const y = (this.yAxisElement)
            ? await this.hydrateYAxis(dims, flattenedData, svg)
            : ChartService.addYAxis(svg, dims, flattenedData, this.axisConfigs);
        if (this.lines) {
            await this.hydrateLines(svg, data, x, y, dims);
        }
        const chartOverlay = ChartService.addOverlay(svg, dims);
        this.addEvents(svg, chartOverlay, x, data);
        ChartService.addRevealAnimation(svg, dims, this.axisConfigs);
        this.appendSVG(svg);
        return Promise.resolve(svg);
    }
    getDefaultXAxisConfig() {
        return {
            showAxis: false,
            labelType: 'date',
        };
    }
    getDefaultYAxisConfig() {
        return {
            showAxis: false,
            labelType: 'number',
        };
    }
    getDims() {
        const margin = { top: 20, right: 20, bottom: 20, left: 20 };
        const { height, width } = this.chartContainer.getBoundingClientRect();
        return {
            height: height - margin.top - margin.bottom,
            width: width - margin.left - margin.right,
            margin
        };
    }
    handleDataSource() {
        let formattedData;
        if (Array.isArray(this.dataSource[0])) {
            this.service = new MultiLineSeries();
            formattedData = this.dataSource.map(d => ChartDataService.formatData(this.axisConfigs, d));
        }
        else {
            this.service = new SingleLineSeries();
            formattedData = ChartDataService.formatData(this.axisConfigs, this.dataSource);
        }
        this.multiSeries = this.service.isMultiSeries();
        this.popover.placement = this.service.getPopoverPlacement();
        return formattedData;
    }
    async hydrateLines(svg, data, x, y, dims) {
        this.seriesMeta = [];
        this.lines.forEach(async (line, i) => {
            const index = (!isNaN(line.dataIndex)) ? line.dataIndex : i;
            const lineData = (this.service.isMultiSeries()) ? data[index] : data;
            const color = await line.init(svg, x, y, lineData, index, dims, this.service.isMultiSeries());
            this.seriesMeta[index] = { name: line.name, color, netNegative: line.netNegative };
        });
    }
    hydrateXAxis(dims, data, svg) {
        return this.xAxisElement.init(svg, dims, data, this.axisConfigs);
    }
    hydrateYAxis(dims, data, svg) {
        return this.yAxisElement.init(svg, dims, data, this.axisConfigs);
    }
    loadLines() {
        return Promise.all(Array.from(this.chartContainer.querySelectorAll('anj-line')).map(col => col.componentOnReady()));
    }
    loadXAxis() {
        return Promise.resolve(this.chartContainer.querySelector('anj-axis[dimension="x"]').componentOnReady());
    }
    loadYAxis() {
        return Promise.resolve(this.chartContainer.querySelector('anj-axis[dimension="y"]').componentOnReady());
    }
    hostData() {
        return {
            class: {
                'anj-multi': this.service.isMultiSeries(),
                'anj-single': !this.service.isMultiSeries()
            }
        };
    }
    render() {
        return [
            h("anj-overlay", { class: "popover", placement: "top", offset: "0, 15", ref: r => this.popover = r },
                h("anj-arrow", null),
                h("slot", null))
        ];
    }
    static get is() { return "anj-line-chart"; }
    static get properties() { return {
        "chartContainer": {
            "elementRef": true
        },
        "dataSource": {
            "type": "Any",
            "attr": "data-source",
            "mutable": true,
            "watchCallbacks": ["dataSourceHandler"]
        },
        "multiSeries": {
            "state": true
        }
    }; }
    static get events() { return [{
            "name": "anjHover",
            "method": "anjHover",
            "bubbles": true,
            "cancelable": true,
            "composed": true
        }]; }
    static get listeners() { return [{
            "name": "anjLineDidLoad",
            "method": "onLinesLoaded"
        }, {
            "name": "anjXAxisDidLoad",
            "method": "onXAxisLoaded"
        }, {
            "name": "anjYAxisDidLoad",
            "method": "onYAxisLoaded"
        }]; }
    static get style() { return "anj-line-chart{display:block;height:100%;position:relative;width:100%}anj-line-chart .overlay{cursor:default;fill:none;pointer-events:all;position:relative;z-index:3}anj-line-chart svg .area{opacity:.1}anj-line-chart svg .axis{font-size:.5rem}anj-line-chart svg .axis line,anj-line-chart svg .axis path{fill:none;stroke:var(--anj-border);shape-rendering:crispEdges}anj-line-chart svg .axis.y path{stroke:transparent}anj-line-chart svg .tick text,anj-line-chart svg text{fill:var(--anj-neutral-light)}anj-line-chart svg .line{fill:none;stroke-width:2px;-webkit-transition:opacity .5s ease,stroke-width .5s ease;transition:opacity .5s ease,stroke-width .5s ease}anj-line-chart svg .overlay{fill:none;pointer-events:all;position:relative;z-index:3}anj-line-chart svg .point{fill:var(--anj-light);stroke-width:1.5px}anj-line-chart.anj-multi anj-arrow{display:none}anj-line-chart .popover{pointer-events:none}"; }
}
LineChart.creationDebounceLength = 50;
LineChart.popoverDebounceLength = 25;

export { Axis as AnjAxis, Line as AnjLine, LineChart as AnjLineChart };
