<p align="center">
<img src="//unpkg.com/@anjuna/theme@0.1.0/assets/anjuna/anjuna-logo.svg"/>
<br/><br/>
</p>

# @anjuna/charts
Authored with Stencil.js, @anjuna/charts is exported as native standard web components and can be used in any UI application.

## Getting Started
To get started with Anjuna, there are a few things to get set up.

## Peer Dependency
Charts relies on @anjuna/core, so be sure to follow that setup before pulling @anjuna/charts in!

### Style
While using Bootstrap under the hood, Anjuna has no runtime dependency on global style outside the theme. Anjuna uses a runtime palette set by the consuming
application, so it will always be consistent with application. The theme can be included by either adding it to the application HTML or CSS.

```html
<!-- HTML -->
<link href="//unpkg.com/@anjuna/theme@0.1.0/css/theme.css" rel="stylesheet">
<link href="//unpkg.com/@anjuna/theme@0.1.0/css/icons.css" rel="stylesheet">
```

```css
/* CSS */
@import url('//unpkg.com/@anjuna/theme@0.1.0/css/theme.css');
@import url('//unpkg.com/@anjuna/theme@0.1.0/css/icons.css');
```

### Loading Components
To use the components in your application, you first need to include them. Depending on your integration, there are a couple ways to do this.

#### Angular
The first step is to install the package into your application:

```
npm install @anjuna/charts
```

The preferred way to add web component libraries to Angular applications is to do so when bootstrapping the app. Update your application's
`main.ts` to add the following functionality:

```typescript
import {enableProdMode} from '@angular/core';
import {platformBrowserDynamic} from '@angular/platform-browser-dynamic';
// import the loader and alias it so it won't conflict with other libraries
import {defineCustomElements as defineAnjunaElements} from '@anjuna/charts/loader';

import {AppModule} from '@app/app.module';
import {environment} from '@env/environment';

if (environment.production) {
    enableProdMode();
}

platformBrowserDynamic().bootstrapModule(AppModule)
    .catch(err => console.log(err));
// Call the define function
defineAnjunaElements(window);
```

##### Custom Elements Schema
By default, Angular does not like it when it doesn't recognize a component tag and will throw an error. To configure Angular modules to tell them
we're using custom elements, we need to add the CUSTOM_ELEMENT_SCHEMA schema to modules that will be making use of them.

```typescript
import {NgModule, CUSTOM_ELEMENTS_SCHEMA} from '@angular/core';

@NgModule({
    declarations: [
        // removed for brevity
    ],
    imports: [
        // removed for brevity
    ],
    schemas: [
        // the important part
        CUSTOM_ELEMENTS_SCHEMA
    ],
    bootstrap: [AppComponent]
})
export class AppModule {}
```

#### Vanilla Javascript
For vanilla apps, the best way to use the components is to include them via CDN in the header of your index.html file.

```html
<!-- Load the components -->
<script src="//unpkg.com/@anjuna/charts@1.0.0-preview.0/anjuna.js"></script>
```
